/*
 * Example1103P
 * Author: Akinori Tsuji
 */
import se.goransson.mqtt.*;
import processing.serial.*;

MQTT mqtt;
Serial myPort;

void setup() {
  myPort = new Serial(this, "COM5", 19200); // COMn を環境に合わせて書き換える
  myPort.bufferUntil('\n');

  mqtt = new MQTT( this );
  mqtt.connect( "web.is.tokushima-u.ac.jp", 1883, "mosquitto-unique-id" );
}


volatile int received = 0; // データの受信が完了した

float temp_max = -40; // 最低気温をセット

void draw() {
  if (received == 1) { // 5秒毎にデータ受信
    float tm = TM;
    if (tm > temp_max) { // 最大値を超えていたら
      temp_max = tm;
      print("temperature current(max):");
      println(temp_max);
    }
    received = 0;
  }
}

//--------------------------------------------------------------------------------------------
// 浮動小数点の数値numを，小数点以下，下n桁に変換
float floatFormat(float num, int n ) {
  float _pow = pow( 10, n ) ;
  return round( num * _pow ) / _pow ;
}

//--------------------------------------------------------------------------------------------
// シリアル通信によって得られたデータの処理（シリアル受信割り込み）
volatile String ED;
volatile float ID, BA, A1, A2, TM, HM, AT;
volatile int SW;
volatile int LX;

void serialEvent(Serial myPort) {
  String myString = myPort.readStringUntil('\n'); // 改行コードまでデータ（文字列）を読み込む
  // 文字列が存在していたら
  if (myString != null) {
    myString = trim(myString); // 空白が含まれていたら削除
//    println(myString);
    String Data1[] = split(myString, ':'); // ':'で文字列を分離して，Data1に格納する
    // データ長（要素）が９以上であれば
    if (Data1.length >= 9) {
      // 文字列Data1の各要素を'='で分離して，各要素を確認する
      String ed[] = split(Data1[1], '='); // 子機の識別番号（MACアドレス下8桁）
      if (ed[0].equals("ed")) {
        ED = ed[1];
      }      
      String id[] = split(Data1[2], '='); // 子機のID番号
      if (id[0].equals("id")) {
        ID = float(id[1]);
      }    
      String ba[] = split(Data1[3], '='); // 電源電圧（電池の電圧）
      if (ba[0].equals("ba")) {
        BA = float(ba[1]);
      }
      String a1[] = split(Data1[4], '='); // スイッチの電圧(mV)
      if (a1[0].equals("a1")) {
        A1 = float(a1[1]); // 0-2400 mV
        if (A1 > 1000) { // スイッチの電圧が1000mV以下でスイッチON
          SW = 0; // OFF
        } else {
          SW = 1; // ON
        }
      }
      String a2[] = split(Data1[5], '='); // 照度センサの電圧(mV)
      if (a2[0].equals("a2")) {
        A2 = float(a2[1]); // 0-2400 mV
        //          LX = 2,22 * (A2 * 1000 / R); // A2 * 1000 [uV], R = 10,000 [Ohm]
        //        LX = 2.22 * (A2 * 1000.0/ 10000.0);
        LX = (int)(0.222 * A2); // lux (0[lux] - 532.8[lux])
      }      
      String tm[] = split(Data1[6], '='); // 温度 * 100 (度)
      if (tm[0].equals("tm")) {
        TM = float(tm[1]) / 100;
      }
      String hm[] = split(Data1[7], '='); // 湿度 * 100 (%RH)
      if (hm[0].equals("hu")) {
        HM = float(hm[1]) / 100;
      }
      String at[] = split(Data1[8], '='); // 大気圧 (hPa)
      if (at[0].equals("at")) {
        AT = float(at[1]);
      }

      //println(ED);
      //println(ID);
      //println(BA);
      //println(SW);
      //println(LX);
      //println(TM);
      //println(HM);
      //println(AT);
      //println();

      // MQTTで送信するメッセージをJSON形式で作成する
      String message = "{";
             message += "\"temp\": "  + str(TM) + ", ";
             message += "\"humid\": " + str(HM) + ", ";
             message += "\"alt\": "   + str(AT) + ", ";
             message += "\"sw\": "    + str(SW) + ", ";
             message += "\"lux\": "   + str(LX);
             message += "}";


      // MATTサーバにトピック(/koukai/teacher/02/temp)をつけてメッセージを発行する
      mqtt.publish("/koukai/teacher/02/temp", message);

      received = 1; // データ受信完了，MQTT送信完了

      myPort.clear();
    }
  }
}